import argparse


get_GTDB_taxon_gnm_parser_usage = '''
================================== get_GTDB_taxon_gnm example commands ==================================

# Example commands
BioSAK get_GTDB_taxon_gnm -meta bac120_metadata_r202.tsv -path genome_paths.tsv -taxon taxons.txt
BioSAK get_GTDB_taxon_gnm -meta ar122_metadata_r202.tsv -path genome_paths.tsv -taxon taxons.txt

# genome metadata: bac120_metadata_r202.tsv or ar122_metadata_r202.tsv
# genome paths: in the fastani folder from auxillary_files
# example of interested taxon file (one taxon per line):
p__Thermoproteota
c__Methanosarcinia
f__Thalassarchaeaceae

=====================================================================================================
'''


def get_GTDB_taxon_gnm(args):

    gtdb_gnm_metadata =         args['meta']
    gtdb_ref_gnm_path_txt =     args['path']
    taxons_to_retrieve_txt =    args['taxon']
    output_prefix =             args['p']

    gnms_to_retrieve_gtdb_txt = '%s_in_GTDB.txt' % output_prefix
    gnms_to_retrieve_ncbi_txt = '%s_in_NCBI.txt' % output_prefix

    # read in taxons_to_retrieve_txt
    taxons_to_retrieve_set = set()
    for each_taxon in open(taxons_to_retrieve_txt):
        taxons_to_retrieve_set.add(each_taxon.strip())

    # read in gtdb_ref_gnm_path_txt
    gtdb_ref_gnm_set = set()
    for each_gnm_path in open(gtdb_ref_gnm_path_txt):
        gnm_path_split = each_gnm_path.strip().split(' ')
        gnm_accession = gnm_path_split[0].split('_genomic')[0]
        gtdb_ref_gnm_set.add(gnm_accession)

    # get genomes from interested taxons
    gnm_set_to_retrieve = set()
    col_index = {}
    for each_ref in open(gtdb_gnm_metadata):
        each_ref_split = each_ref.strip().split('\t')
        if each_ref.startswith('accession	ambiguous_bases'):
            col_index = {key: i for i, key in enumerate(each_ref_split)}
        else:
            ref_accession = each_ref_split[0][3:]
            gtdb_taxonomy = each_ref_split[col_index['gtdb_taxonomy']]
            for each_interested_taxon in taxons_to_retrieve_set:
                if each_interested_taxon in gtdb_taxonomy:
                    gnm_set_to_retrieve.add(ref_accession)

    # write out
    from_gtdb = 0
    from_ncbi = 0
    gnms_to_retrieve_gtdb_txt_handle = open(gnms_to_retrieve_gtdb_txt, 'w')
    gnms_to_retrieve_ncbi_txt_handle = open(gnms_to_retrieve_ncbi_txt, 'w')
    for each_gnm_to_retrieve in sorted([i for i in gnm_set_to_retrieve]):
        if each_gnm_to_retrieve in gtdb_ref_gnm_set:
            gnms_to_retrieve_gtdb_txt_handle.write('%s\n' % each_gnm_to_retrieve)
            from_gtdb += 1
        else:
            gnms_to_retrieve_ncbi_txt_handle.write('%s\n' % each_gnm_to_retrieve)
            from_ncbi += 1
    gnms_to_retrieve_gtdb_txt_handle.close()
    gnms_to_retrieve_ncbi_txt_handle.close()

    print('Total number of genomes from interested taxons: %s' % len(gnm_set_to_retrieve))
    print('Genomes in GTDB: %s, see details in %s'   % (from_gtdb, gnms_to_retrieve_gtdb_txt))
    print('Genomes in NCBI: %s, see details in %s' % (from_ncbi, gnms_to_retrieve_ncbi_txt))


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-meta',        required=True,              help='GTDB reference genome metadata')
    parser.add_argument('-path',        required=True,              help='GTDB genome paths')
    parser.add_argument('-taxon',       required=True,              help='interested taxons, one taxon per line')
    parser.add_argument('-p',           required=False, default='', help='output prefix')
    args = vars(parser.parse_args())
    get_GTDB_taxon_gnm(args)



import argparse
from Bio import AlignIO
from Bio.Alphabet import IUPAC, Gapped
from Bio.Align import MultipleSeqAlignment


SubsetAlnCols_usage = '''
========================= SubsetAlnCols example commands =========================

BioSAK SubsetAlnCols -in 16S.aln -r 200-300 -out 16S_v1.aln
BioSAK SubsetAlnCols -in 16S.aln -r 200-300 -pct 80 -out 16S_v1.aln

===============================================================================
'''


def SubsetAlnCols(args):

    aln_in_fasta      = args['in']
    col_range         = args['r']
    min_non_empty_pct = args['pct']
    align_subset_out  = args['out']


    # get range
    col_start        = int(col_range.split('-')[0])
    col_end          = int(col_range.split('-')[1])
    align_subset_len = col_end - col_start + 1

    # subset
    align_in         = AlignIO.read(aln_in_fasta, 'fasta')
    align_in_subset  = align_in[:, (col_start - 1):col_end]

    if min_non_empty_pct is None:

        # write out
        align_file_out_handle = open(align_subset_out, 'w')
        AlignIO.write(align_in_subset, align_file_out_handle, 'fasta')
        align_file_out_handle.close()

    else:

        # filter
        align_in_subset_filtered = MultipleSeqAlignment([], Gapped(IUPAC.unambiguous_dna, "-"))
        for seq_record in align_in_subset:
            non_empty_col_pct = (align_subset_len - seq_record.seq.count('-'))*100/align_subset_len
            if non_empty_col_pct >= float(min_non_empty_pct):
                align_in_subset_filtered.add_sequence(seq_record.id, str(seq_record.seq))

        # write out
        align_file_out_handle = open(align_subset_out, 'w')
        AlignIO.write(align_in_subset_filtered, align_file_out_handle, 'fasta')
        align_file_out_handle.close()


if __name__ == '__main__':

    SubsetAlnCols_parser = argparse.ArgumentParser()

    # arguments for rename_seq_parser
    SubsetAlnCols_parser.add_argument('-in',    required=True,                help='input MSA in fasta format')
    SubsetAlnCols_parser.add_argument('-r',     required=True,                help='columns to keep, e.g. 200-300, one based')
    SubsetAlnCols_parser.add_argument('-pct',   required=False, default=None, help='minimum percentage of nonempty bases (e.g. 70), default keep all')
    SubsetAlnCols_parser.add_argument('-out',   required=True,                help='output file')

    args = vars(SubsetAlnCols_parser.parse_args())

    SubsetAlnCols(args)




import os
import glob
import shutil
import argparse
from Bio import SeqIO
from Bio.Seq import Seq
from Bio.Alphabet import IUPAC
from Bio import SeqFeature as SF
from Bio.SeqRecord import SeqRecord
from Bio.Alphabet import generic_dna
from Bio.SeqFeature import SeqFeature
from Bio.SeqFeature import FeatureLocation
import multiprocessing as mp
from datetime import datetime


Prodigal_parser_usage = '''
==================== Prodigal_Runner example commands ====================

# for completed genome
BioSAK Prodigal_Runner -i genome_folder -x fa -p KelpGenome -t 6

# for metagenome-assembled genomes (MAGs) 
BioSAK Prodigal_Runner -i bin_folder -x fa -p KelpBins -t 6 -meta

Software dependencies:
module load python/3.5.2
module load prodigal/2.6.3 

==========================================================================
'''


def force_create_folder(folder_to_create):
    if os.path.isdir(folder_to_create):
        shutil.rmtree(folder_to_create)
        if os.path.isdir(folder_to_create):
            shutil.rmtree(folder_to_create)
            if os.path.isdir(folder_to_create):
                shutil.rmtree(folder_to_create)
                if os.path.isdir(folder_to_create):
                    shutil.rmtree(folder_to_create)
    os.mkdir(folder_to_create)


def export_dna_record(gene_seq, gene_id, gene_description, output_handle):
    seq_object = Seq(gene_seq, IUPAC.unambiguous_dna)
    seq_record = SeqRecord(seq_object)
    seq_record.id = gene_id
    seq_record.description = gene_description
    SeqIO.write(seq_record, output_handle, 'fasta')


def export_aa_record(gene_seq, gene_id, gene_description, output_handle):
    seq_object = Seq(gene_seq, IUPAC.protein)
    seq_record = SeqRecord(seq_object)
    seq_record.id = gene_id
    seq_record.description = gene_description
    SeqIO.write(seq_record, output_handle, 'fasta')


def prodigal_parser(seq_file, sco_file, prefix, output_folder_ffn, output_folder_faa, output_folder_gbk):

    bin_ffn_file =     '%s.ffn' % prefix
    bin_faa_file =     '%s.faa' % prefix
    bin_gbk_file =     '%s.gbk' % prefix
    pwd_bin_ffn_file = '%s/%s'  % (output_folder_ffn, bin_ffn_file)
    pwd_bin_faa_file = '%s/%s'  % (output_folder_faa, bin_faa_file)
    pwd_bin_gbk_file = '%s/%s'  % (output_folder_gbk, bin_gbk_file)

    # get sequence id list
    id_to_sequence_dict = {}
    sequence_id_list = []
    for each_seq in SeqIO.parse(seq_file, 'fasta'):
        id_to_sequence_dict[each_seq.id] = str(each_seq.seq)
        sequence_id_list.append(each_seq.id)


    # get sequence to cds dict and sequence to transl_table dict
    current_seq_id = ''
    current_transl_table = ''
    current_seq_csd_list = []
    seq_to_cds_dict = {}
    seq_to_transl_table_dict = {}
    for each_cds in open(sco_file):
        if each_cds.startswith('# Sequence Data'):

            # add to dict
            if current_seq_id != '':
                seq_to_cds_dict[current_seq_id] = current_seq_csd_list
                seq_to_transl_table_dict[current_seq_id] = current_transl_table

            # reset value
            current_seq_id = each_cds.strip().split('=')[-1][1:-1].split(' ')[0]
            current_transl_table = ''
            current_seq_csd_list = []

        elif each_cds.startswith('# Model Data'):
            current_transl_table = each_cds.strip().split(';')[-2].split('=')[-1]

        else:
            current_seq_csd_list.append('_'.join(each_cds.strip().split('_')[1:]))

    seq_to_cds_dict[current_seq_id] = current_seq_csd_list
    seq_to_transl_table_dict[current_seq_id] = current_transl_table


    bin_gbk_file_handle = open(pwd_bin_gbk_file, 'w')
    bin_ffn_file_handle = open(pwd_bin_ffn_file, 'w')
    bin_faa_file_handle = open(pwd_bin_faa_file, 'w')
    gene_index = 1
    for seq_id in sequence_id_list:

        # create SeqRecord
        current_sequence = Seq(id_to_sequence_dict[seq_id])
        current_SeqRecord = SeqRecord(current_sequence, id=seq_id)
        current_SeqRecord.seq.alphabet = generic_dna
        transl_table = seq_to_transl_table_dict[seq_id]

        current_SeqRecord_annotations = {}
        # current_SeqRecord_annotations['data_file_division'] =   'UNA'
        current_SeqRecord_annotations['date'] =                 (datetime.now().strftime('%d-%b-%Y')).upper()
        # current_SeqRecord_annotations['definition'] =           '%s_%s' % (prefix, seq_id)
        current_SeqRecord_annotations['accession'] =            ''
        current_SeqRecord_annotations['version'] =              ''
        current_SeqRecord_annotations['keywords'] =             ['.']
        current_SeqRecord_annotations['source'] =               prefix
        current_SeqRecord_annotations['organism'] =             prefix
        current_SeqRecord_annotations['taxonomy'] =             ['Unclassified']
        current_SeqRecord_annotations['comment'] =              '.'
        current_SeqRecord.annotations = current_SeqRecord_annotations


        # add SeqFeature to SeqRecord
        for cds in seq_to_cds_dict[seq_id]:

            # define locus_tag id
            locus_tag_id = '%s_%s' % (prefix, "{:0>5}".format(gene_index))

            # define FeatureLocation
            cds_split = cds.split('_')
            cds_start = SF.ExactPosition(int(cds_split[0]))
            cds_end = SF.ExactPosition(int(cds_split[1]))
            cds_strand = cds_split[2]
            current_strand = None
            if cds_strand == '+':
                current_strand = 1
            if cds_strand == '-':
                current_strand = -1
            current_feature_location = FeatureLocation(cds_start, cds_end, strand=current_strand)

            # get nc sequence
            sequence_nc = ''
            if cds_strand == '+':
                sequence_nc = id_to_sequence_dict[seq_id][cds_start-1:cds_end]
            if cds_strand == '-':
                sequence_nc = str(Seq(id_to_sequence_dict[seq_id][cds_start-1:cds_end], generic_dna).reverse_complement())

            # translate to aa sequence
            sequence_aa = str(SeqRecord(Seq(sequence_nc)).seq.translate(table=transl_table))

            # remove * at the end
            sequence_aa = sequence_aa[:-1]

            # export nc and aa sequences
            export_dna_record(sequence_nc, locus_tag_id, '', bin_ffn_file_handle)
            export_aa_record(sequence_aa, locus_tag_id, '', bin_faa_file_handle)

            # Define feature type
            current_feature_type = 'CDS'

            # Define feature qualifiers
            current_qualifiers_dict = {}
            current_qualifiers_dict['locus_tag'] = locus_tag_id
            current_qualifiers_dict['transl_table'] = transl_table
            current_qualifiers_dict['translation'] = sequence_aa

            # Create a SeqFeature
            current_feature = SeqFeature(current_feature_location, type=current_feature_type, qualifiers=current_qualifiers_dict)

            # Append Feature to SeqRecord
            current_SeqRecord.features.append(current_feature)
            gene_index += 1

        # export to gbk file
        SeqIO.write(current_SeqRecord, bin_gbk_file_handle, 'genbank')

    bin_ffn_file_handle.close()
    bin_faa_file_handle.close()
    bin_gbk_file_handle.close()


def prodigal_worker(argument_list):

    input_genome = argument_list[0]
    input_genome_folder = argument_list[1]
    meta_mode = argument_list[2]
    pwd_prodigal_output_folder = argument_list[3]
    pwd_prodigal_output_ffn_folder = argument_list[4]
    pwd_prodigal_output_faa_folder = argument_list[5]
    pwd_prodigal_output_gbk_folder = argument_list[6]

    # prepare command (according to Prokka)
    input_genome_basename, input_genome_ext = os.path.splitext(input_genome)
    pwd_input_genome = '%s/%s' % (input_genome_folder, input_genome)
    pwd_output_sco = '%s/%s.sco' % (pwd_prodigal_output_folder, input_genome_basename)

    prodigal_cmd_meta = 'prodigal -f sco -q -c -m -g 11 -p meta -i %s -o %s' % (pwd_input_genome, pwd_output_sco)
    prodigal_cmd_nonmeta = 'prodigal -f sco -q -c -m -g 11 -i %s -o %s' % (pwd_input_genome, pwd_output_sco)

    if meta_mode is True:
        prodigal_cmd = prodigal_cmd_meta
    else:
        prodigal_cmd = prodigal_cmd_nonmeta

    os.system(prodigal_cmd)

    # prepare ffn, faa and gbk files from prodigal output
    prodigal_parser(pwd_input_genome, pwd_output_sco, input_genome_basename, pwd_prodigal_output_ffn_folder, pwd_prodigal_output_faa_folder, pwd_prodigal_output_gbk_folder)


def Annotation_Prodigal(args):

    input_genome_folder = args['i']
    file_extension = args['x']
    output_prefix = args['p']
    meta_mode = args['meta']
    num_threads = args['t']

    # create output folder
    output_folder_sco = '%s_prodigal_sco' % output_prefix
    output_folder_ffn = '%s_prodigal_ffn' % output_prefix
    output_folder_faa = '%s_prodigal_faa' % output_prefix
    output_folder_gbk = '%s_prodigal_gbk' % output_prefix

    force_create_folder(output_folder_sco)
    force_create_folder(output_folder_ffn)
    force_create_folder(output_folder_faa)
    force_create_folder(output_folder_gbk)

    # get input genome list
    input_genome_re = '%s/*.%s' % (input_genome_folder, file_extension)
    input_genome_file_list = [os.path.basename(file_name) for file_name in glob.glob(input_genome_re)]

    # prepare command list
    list_for_multiple_arguments_Prodigal = []
    for input_genome in input_genome_file_list:
        list_for_multiple_arguments_Prodigal.append([input_genome, input_genome_folder, meta_mode, output_folder_sco, output_folder_ffn, output_folder_faa, output_folder_gbk])

    # run prodigal with multiprocessing
    pool = mp.Pool(processes=num_threads)
    pool.map(prodigal_worker, list_for_multiple_arguments_Prodigal)
    pool.close()
    pool.join()


if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(help="--", dest='subparser_name')

    # Annotation modules
    Prodigal_parser = subparsers.add_parser('Prodigal_Runner', description='Wrapper for running Prodigal',usage=Prodigal_parser_usage)

    Prodigal_parser.add_argument('-i',          required=True,  help='input genome folder')
    Prodigal_parser.add_argument('-x',          required=False, default='fasta', help='file extension')
    Prodigal_parser.add_argument('-p',          required=True,  help='output prefix')
    Prodigal_parser.add_argument('-meta',       required=False, action="store_true", help='annotation mode for metagenome assembled genomes (MAGs)')
    Prodigal_parser.add_argument('-t',          required=False, type=int, default=1, help='number of threads')

    args = vars(parser.parse_args())

    Annotation_Prodigal(args)



import argparse
import networkx as nx
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
from datetime import datetime
from BioSAK.BioSAK_config import config_dict
from BioSAK.global_functions import is_number
from BioSAK.global_functions import time_format
from BioSAK.global_functions import sep_path_basename_ext


NetEnzymes_parser_usage = '''
=========================== NetEnzymes example commands ===========================

# get network of all enzymes in Ecoli_ec.txt
BioSAK NetEnzymes -ec Ecoli_ec.txt -to_skip skip.txt -NoHyphen

# get network of enzymes belong to ko 00010 (Glycolysis) in Ecoli_ec.txt
BioSAK NetEnzymes -ec Ecoli_ec.txt -ko 00010 -to_skip skip.txt -NoHyphen -plot 

# EC file format (one EC per line):
2.7.7.23
3.6.3.28

# to_skip file format (one substrate per line):
H2O
H+

===================================================================================
'''

'''
cd /Users/songweizhi/MetaCyc_demo
python3 ~/PycharmProjects/BioSAK/BioSAK/NetEnzymes.py -ec Ecoli_ec.txt -to_skip skip.txt -plot -NoHyphen
python3 ~/PycharmProjects/BioSAK/BioSAK/NetEnzymes.py -ec Ecoli_ec.txt -ko 00010 -to_skip skip.txt -plot -NoHyphen
python3 ~/PycharmProjects/BioSAK/BioSAK/NetEnzymes.py -ec Ecoli_ec.txt -ko 00020 -to_skip skip.txt -plot -NoHyphen
python3 ~/PycharmProjects/BioSAK/BioSAK/NetEnzymes.py -ec Glycolysis_ECs.txt -to_skip skip.txt -NoHyphen -plot 

python3 ~/PycharmProjects/BioSAK/BioSAK/NetEnzymes.py -enzymes TCA_enzymes.txt -compounds TCA_compounds.txt -plot
enzymes 

BioSAK NetEnzymes -ec Ecoli_ec.txt -ko 00020 -to_skip skip.txt -plot -NoHyphen
BioSAK NetEnzymes -ec Glycolysis_ECs.txt -to_skip skip.txt -NoHyphen -plot

To-do:
1. Provide a network for a list of enzymes and substrates
2. '=' symbol

'''


def get_ko2description_dict(ko00001_keg):

    As_description_dict = {}
    Bs_description_dict = {}
    Cs_description_dict = {}
    Ds_description_dict = {}
    D2ABCD_dict = {}
    ko2level_dict = {}

    current_A = ''
    current_B = ''
    current_C = ''
    for each_line in open(ko00001_keg):
        if each_line[0] in ['A', 'B', 'C', 'D']:
            each_line_split = each_line.strip().split(' ')

            if each_line[0] == 'A':
                current_A_id = each_line_split[0]
                current_A_description = ' '.join(each_line_split[1:-1])
                ko2level_dict[current_A_id] = 'A'
                current_A = current_A_id
                As_description_dict[current_A_id] = current_A_description

            elif each_line[0] == 'B':
                if len(each_line_split) > 1:
                    current_B_id = each_line_split[2]
                    current_B_description = ' '.join(each_line_split[3:-1])
                    ko2level_dict[current_B_id] = 'B'
                    current_B = current_B_id
                    Bs_description_dict[current_B_id] = current_B_description

            elif each_line[0] == 'C':
                current_C_id = each_line_split[4]
                current_C_description = ' '.join(each_line_split[5:-1])
                ko2level_dict[current_C_id] = 'C'
                current_C = current_C_id
                Cs_description_dict[current_C_id] = current_C_description

            elif each_line[0] == 'D':
                current_D_id = each_line_split[6]
                current_D_description = ' '.join(each_line_split[7:])
                ko2level_dict[current_D_id] = 'D'
                Ds_description_dict[current_D_id] = current_D_description
                ABCD_value = '%s|%s|%s|%s' % (current_A, current_B, current_C, current_D_id)
                if current_D_id not in D2ABCD_dict:
                    D2ABCD_dict[current_D_id] = [ABCD_value]
                elif (current_D_id in D2ABCD_dict) and (ABCD_value not in D2ABCD_dict[current_D_id]):
                    D2ABCD_dict[current_D_id].append(ABCD_value)

    return As_description_dict, Bs_description_dict, Cs_description_dict, Ds_description_dict, D2ABCD_dict, ko2level_dict


def get_ec_of_interested_ko(D2ABCD_dict, KO_description_D_dict, ko_level, ko_id):

    # get ec list
    interested_ko_ec_list = set()
    for each_ko_D in D2ABCD_dict:

        each_ko_D_description = KO_description_D_dict[each_ko_D]
        if '[EC:' in each_ko_D_description:
            ec_str = each_ko_D_description.strip().split('[EC:')[1][:-1]

            # get ec list
            ec_list = [ec_str]
            if ' ' in ec_str:
                ec_list = ec_str.split(' ')

            for each_group in D2ABCD_dict[each_ko_D]:
                each_ko_D_split = each_group.split('|')
                if (ko_level == 'A') and (each_ko_D_split[0] == ko_id):
                    for ec in ec_list:
                        interested_ko_ec_list.add(ec)

                elif (ko_level == 'B') and (each_ko_D_split[1] == ko_id):
                    for ec in ec_list:
                        interested_ko_ec_list.add(ec)

                elif (ko_level == 'C') and (each_ko_D_split[2] == ko_id):
                    for ec in ec_list:
                        interested_ko_ec_list.add(ec)

                elif (ko_level == 'D') and (each_ko_D_split[3] == ko_id):
                    for ec in ec_list:
                        interested_ko_ec_list.add(ec)

    return interested_ko_ec_list


def parse_biological_raction(G, reaction, compounds_to_exclude, compounds_to_include, node_color_dict):

    reaction_split = reaction.strip().split('\t')
    enzyme = reaction_split[0]
    reaction_equation = reaction_split[1]

    G.add_node(enzyme, data=True, shape='s', color_map=node_color_dict['enzyme'])

    substrates = ''
    products = ''
    if ' → ' in reaction_equation:
        reaction_equation_split = reaction_equation.split(' → ')
        substrates = reaction_equation_split[0]
        products = reaction_equation_split[1]
    elif ' ↔ ' in reaction_equation:
        reaction_equation_split = reaction_equation.split(' ↔ ')
        substrates = reaction_equation_split[0]
        products = reaction_equation_split[1]
    if ' ← ' in reaction_equation:
        reaction_equation_split = reaction_equation.split(' ← ')
        substrates = reaction_equation_split[1]
        products = reaction_equation_split[0]

    substrate_list = [substrates]
    if '+' in substrates:
        substrate_list = substrates.split(' + ')

    product_list = [products]
    if '+' in products:
        product_list = products.split(' + ')

    substrate_list_no_num = []
    for substrate in substrate_list:
       if (' ' in substrate):
           substrate_split = substrate.split(' ')
           if (is_number(substrate_split[0]) is True) or (substrate_split[0] in ['a', 'an', 'n']):
               substrate_list_no_num.append(' '.join(substrate_split[1:]))
           else:
               substrate_list_no_num.append(substrate)
       else:
           substrate_list_no_num.append(substrate)


    product_list_no_num = []
    for product in product_list:
        if (' ' in product):
            product_split = product.split(' ')
            if (is_number(product_split[0]) is True) or (product_split[0] in ['a', 'an', 'n']):
                product_list_no_num.append(' '.join(product_split[1:]))
            else:
                product_list_no_num.append(product)
        else:
            product_list_no_num.append(product)

    # remove brackets
    substrate_list_no_num_brackets = []
    for substrate_no_num in substrate_list_no_num:
        if '[' in substrate_no_num:

            substrate_no_num_split = substrate_no_num.split('[')
            substrate_no_num_brackets = substrate_no_num_split[0]
            substrate_list_no_num_brackets.append(substrate_no_num_brackets)
        else:
            substrate_list_no_num_brackets.append(substrate_no_num)

    #print(substrate_list_no_num)
    #print(substrate_list_no_num_brackets)

    product_list_no_num_brackets = []
    for product_no_num in product_list_no_num:
        if '[' in product_no_num:
            product_no_num_split = product_no_num.split('[')
            product_no_num_brackets = product_no_num_split[0]
            product_list_no_num_brackets.append(product_no_num_brackets)
        else:
            product_list_no_num_brackets.append(product_no_num)

    for substrate in substrate_list_no_num_brackets:
        if substrate not in compounds_to_exclude:

            if compounds_to_include == 'all':
                G.add_node(substrate, data=True, shape='o', color_map=node_color_dict['substrate'])
                G.add_edge(substrate, enzyme)
                if '↔' in reaction_equation:
                    G.add_edge(enzyme, substrate)
            else:
                if substrate in compounds_to_include:
                    G.add_node(substrate, data=True, shape='o', color_map=node_color_dict['substrate'])
                    G.add_edge(substrate, enzyme)
                    if '↔' in reaction_equation:
                        G.add_edge(enzyme, substrate)


    for product in product_list_no_num_brackets:
        if product not in compounds_to_exclude:

            if compounds_to_include == 'all':

                G.add_node(product, data=True, shape='o', color_map=node_color_dict['product'])
                G.add_edge(enzyme, product)
                if '↔' in reaction_equation:
                    G.add_edge(product, enzyme)
            else:
                if product in compounds_to_include:
                    G.add_node(product, data=True, shape='o', color_map=node_color_dict['product'])
                    G.add_edge(enzyme, product)
                    if '↔' in reaction_equation:
                        G.add_edge(product, enzyme)


def NetEnzymes(args, config_dict):

    enzymes_list_file           = args['enzymes']
    compound_list_file          = args['compounds']
    interested_ko_id            = args['ko']
    ignore_ec_with_hyphen       = args['NoHyphen']
    to_skip_file                = args['to_skip']
    plot_network                = args['plot']
    label_font_size             = args['lfs']
    node_size                   = args['ns']
    ko00001_keg                 = config_dict['ko00001_keg']
    db_file_with_ec             = config_dict['MetaCyc_rxns_with_ec']

    ########################################################################################################################

    node_color_dict = {'enzyme': 'lightgreen', 'substrate': 'grey', 'product': 'grey'}

    skip_list = set()
    if to_skip_file is not None:
        for each_to_skip in open(to_skip_file):
            skip_list.add(each_to_skip.strip())

    compounds_to_include_list = 'all'
    if compound_list_file is not None:
        compounds_to_include_list = set()
        for compound_to_include in open(compound_list_file):
            compounds_to_include_list.add(compound_to_include.strip())

    # define output file name
    ec_file_no_path, ec_file_no_ext, ec_file_ext = sep_path_basename_ext(enzymes_list_file)

    if interested_ko_id is None:
        if ignore_ec_with_hyphen is True:
            output_graphml  = '%s/%s_NoHyphen.graphml'      % (ec_file_no_path, ec_file_no_ext)
            output_plot     = '%s/%s_NoHyphen.png'          % (ec_file_no_path, ec_file_no_ext)
        else:
            output_graphml  = '%s/%s.graphml'               % (ec_file_no_path, ec_file_no_ext)
            output_plot     = '%s/%s.png'                   % (ec_file_no_path, ec_file_no_ext)
    else:
        if ignore_ec_with_hyphen is True:
            output_graphml  = '%s/%s_ko%s_NoHyphen.graphml' % (ec_file_no_path, ec_file_no_ext, interested_ko_id)
            output_plot     = '%s/%s_ko%s_NoHyphen.png'     % (ec_file_no_path, ec_file_no_ext, interested_ko_id)
        else:
            output_graphml  = '%s/%s_ko%s.graphml'          % (ec_file_no_path, ec_file_no_ext, interested_ko_id)
            output_plot     = '%s/%s_ko%s.png'              % (ec_file_no_path, ec_file_no_ext, interested_ko_id)

    ########################################################################################################################

    interested_ec_list = []
    if interested_ko_id is not None:

        print(datetime.now().strftime(time_format) + 'get ECs from interested KO')

        # read in KEGG db file
        KO_description_A_dict, KO_description_B_dict, KO_description_C_dict, KO_description_D_dict, D2ABCD_dict, ko2level_dict = get_ko2description_dict(ko00001_keg)

        # get ec list from interested KO category
        interested_ec_list = get_ec_of_interested_ko(D2ABCD_dict, KO_description_D_dict, ko2level_dict[interested_ko_id], interested_ko_id)

    # get identified_ec_list
    print(datetime.now().strftime(time_format) + 'read in provided ECs')

    identified_ec_list = set()
    for ec in open(enzymes_list_file):
        ec = ec.strip()
        if interested_ko_id is not None:
            if ec in interested_ec_list:
                if ignore_ec_with_hyphen is False:
                    identified_ec_list.add(ec)
                else:
                    if '-' not in ec:
                        identified_ec_list.add(ec)
        else:
            if ignore_ec_with_hyphen is False:
                identified_ec_list.add(ec)
            else:
                if '-' not in ec:
                    identified_ec_list.add(ec)

    # initialize a graph
    G = nx.DiGraph()

    print(datetime.now().strftime(time_format) + 'add nodes and edges to network')

    # add node and edge
    for reaction in open(db_file_with_ec):
        ec_id = reaction.strip().split('\t')[0]

        if ec_id in identified_ec_list:
            parse_biological_raction(G, reaction, skip_list, compounds_to_include_list, node_color_dict)

    print(datetime.now().strftime(time_format) + 'write out network to graphml file')

    # write out graphml
    nx.write_graphml(G, output_graphml)

    if plot_network is True:

        print(datetime.now().strftime(time_format) + 'plot network')

        # specify
        graph_layout = nx.layout.kamada_kawai_layout(G)  # kamada_kawai_layout, planar_layout, fruchterman_reingold_layout

        # turn node attributes into dict
        node_attributes_dict = {}
        for node in G.nodes(data=True):
            node_attributes_dict[node[0]] = node[1]

        print(datetime.now().strftime(time_format) + 'plot nodes')

        # plot node
        for node in G:
            nx.draw_networkx_nodes(G, graph_layout,
                                   nodelist=[node],
                                   node_size=node_size,
                                   node_color=node_attributes_dict[node]['color_map'],
                                   node_shape=node_attributes_dict[node]['shape'])

            # add customized node label
            # nx.draw_networkx_labels(g, graph_layout, nodelist=[node], font_size=8, font_color='black')

        #  all nodes label together
        nx.draw_networkx_labels(G, graph_layout, nodelist=G.nodes, font_size=label_font_size, font_color='black')

        print(datetime.now().strftime(time_format) + 'plot edges')

        # plot edges
        nx.draw_networkx_edges(G, graph_layout, width=0.5, arrows=True, arrowsize=6)

        # save plot
        plt.savefig(output_plot, dpi=300)
        plt.close()

    ########################################################################################################################

    # G_in_cytoscape_data = json_graph.cytoscape_data(G)
    # print(G_in_cytoscape_data)
    # G_in_cytoscape_graph = json_graph.cytoscape_graph(G_in_cytoscape_data)
    # print(G_in_cytoscape_data)

    print(datetime.now().strftime(time_format) + 'Done!')


if __name__ == '__main__':

    NetEnzymes_parser = argparse.ArgumentParser()

    # arguments for NetMetaCyc_parser
    NetEnzymes_parser.add_argument('-enzymes',   required=True,                           help='Enzyme list file')
    NetEnzymes_parser.add_argument('-compounds', required=False, default=None,            help='compound list file')
    NetEnzymes_parser.add_argument('-ko',        required=False, default=None,            help='get network of enzymes from specified ko')
    NetEnzymes_parser.add_argument('-to_skip',   required=False, default=None,            help='substrates/products to ignore (e.g. H2O, CO2, H+, ATP, ADP)')
    NetEnzymes_parser.add_argument('-NoHyphen',  required=False, action='store_true',     help='ignore enzymes with "-" in EC')
    NetEnzymes_parser.add_argument('-plot',      required=False, action='store_true',     help='plot network, slow and messy layout for complicated network')
    NetEnzymes_parser.add_argument('-lfs',       required=False,  default=3, type=float,  help='Font size of node labels, default is 3')
    NetEnzymes_parser.add_argument('-ns',        required=False, default=20, type=float,  help='Node size, default is 20')

    args = vars(NetEnzymes_parser.parse_args())

    NetEnzymes(args, config_dict)



import os
from Bio import SeqIO
from Bio.Seq import Seq
from Bio.Alphabet import IUPAC
from Bio.SeqRecord import SeqRecord
from Bio.Alphabet import generic_dna


def export_seq_record(gene_seq, gene_id, gene_description, seq_type, output_handle):

    IUPAC_type = None
    if seq_type == 'N':
        IUPAC_type = IUPAC.unambiguous_dna

    if seq_type == 'P':
        IUPAC_type = IUPAC.protein

    seq_object = Seq(gene_seq, IUPAC_type)
    seq_record = SeqRecord(seq_object)
    seq_record.id = gene_id
    seq_record.description = gene_description
    SeqIO.write(seq_record, output_handle, 'fasta')


def export_dna_record(gene_seq, gene_id, gene_description, output_handle):
    seq_object = Seq(gene_seq, IUPAC.unambiguous_dna)
    seq_record = SeqRecord(seq_object)
    seq_record.id = gene_id
    seq_record.description = gene_description
    SeqIO.write(seq_record, output_handle, 'fasta')


def export_aa_record(gene_seq, gene_id, gene_description, output_handle):
    seq_object = Seq(gene_seq, IUPAC.protein)
    seq_record = SeqRecord(seq_object)
    seq_record.id = gene_id
    seq_record.description = gene_description
    SeqIO.write(seq_record, output_handle, 'fasta')


input_gbk_file = '/Users/songweizhi/Desktop/input_gbk_files/NorthSea_bin014.gbk'
export_type = 'A'  # N or P or A


# prepare file name
gbk_file_path, gbk_file_name = os.path.split(input_gbk_file)
gbk_file_name_no_extension, gbk_file_extension = os.path.splitext(gbk_file_name)
output_ffn_filename = '%s.ffn' % gbk_file_name_no_extension
output_faa_filename = '%s.faa' % gbk_file_name_no_extension

if gbk_file_path != '':
    pwd_output_ffn_file = '%s/%s' % (gbk_file_path, output_ffn_filename)
    pwd_output_faa_file = '%s/%s' % (gbk_file_path, output_faa_filename)
else:
    pwd_output_ffn_file = output_ffn_filename
    pwd_output_faa_file = output_faa_filename

output_ffn_handle = None
output_faa_handle = None
if export_type == 'N':
    output_ffn_handle = open(pwd_output_ffn_file, 'w')
if export_type == 'P':
    output_faa_handle = open(pwd_output_faa_file, 'w')
if export_type == 'A':
    output_ffn_handle = open(pwd_output_ffn_file, 'w')
    output_faa_handle = open(pwd_output_faa_file, 'w')

for seq_record in SeqIO.parse(input_gbk_file, 'genbank'):
    for feature in seq_record.features:
        if feature.type == "CDS":
            seq_record_sequence = str(seq_record.seq)

            # get DNA sequence
            seq_nc = ''
            if feature.location.strand == 1:
                seq_nc = seq_record_sequence[feature.location.start:feature.location.end]
            if feature.location.strand == -1:
                nc_seq_rc = seq_record_sequence[feature.location.start:feature.location.end]
                seq_nc = str(Seq(nc_seq_rc, generic_dna).reverse_complement())

            # get aa sequence
            seq_aa = feature.qualifiers['translation'][0]
            feature_id = feature.qualifiers['locus_tag'][0]
            feature_description = feature.qualifiers['product'][0]

            # export to file
            if export_type == 'N':
                export_dna_record(seq_nc, feature_id, feature_description, output_ffn_handle)
            if export_type == 'P':
                export_aa_record(seq_aa, feature_id, feature_description, output_faa_handle)
            if export_type == 'A':
                export_dna_record(seq_nc, feature_id, feature_description, output_ffn_handle)
                export_aa_record(seq_aa, feature_id, feature_description, output_faa_handle)

# close file
if export_type == 'N':
    output_ffn_handle.close()
if export_type == 'P':
    output_faa_handle.close()
if export_type == 'A':
    output_ffn_handle.close()
    output_faa_handle.close()







import os
import shutil
import argparse
from requests import get
import multiprocessing as mp
from datetime import datetime
from bs4 import BeautifulSoup


''' 
cd /Users/songweizhi/Desktop
python3 ~/PycharmProjects/BioSAK/BioSAK/get_ko_gene_seqs.py -ko interested_ko.txt -o test_test -force -t 4
'''


def force_create_folder(folder_to_create):
    if os.path.isdir(folder_to_create):
        shutil.rmtree(folder_to_create, ignore_errors=True)
        if os.path.isdir(folder_to_create):
            shutil.rmtree(folder_to_create, ignore_errors=True)
            if os.path.isdir(folder_to_create):
                shutil.rmtree(folder_to_create, ignore_errors=True)
                if os.path.isdir(folder_to_create):
                    shutil.rmtree(folder_to_create, ignore_errors=True)
    os.mkdir(folder_to_create)


def download_ko_genes(argument_list):

    ko              = argument_list[0]
    fasta_file_out  = argument_list[1]


    ko_url = 'https://www.genome.jp/dbget-bin/www_bget?ko:%s' % ko
    web_address_K18831_soup = BeautifulSoup(get(ko_url).text, features="lxml")

    # get total number of genes
    total_gene_num = 0
    for each_row in web_address_K18831_soup.find_all('tr'):
        try:
            row_name = each_row.th.text
            if row_name == 'Genes':
                gene_list = each_row.find_all('a')
                for each_gene in gene_list:
                    if '/dbget-bin/www_bget?' in each_gene.get('href'):
                        total_gene_num += 1
        except:
            pass

    time_format = '[%Y-%m-%d %H:%M:%S]'
    print('%s Downloading %s genes for %s.' % ((datetime.now().strftime(time_format)), total_gene_num, ko))

    # write out sequences
    file_out_handle = open(fasta_file_out, 'w')
    for each_row in web_address_K18831_soup.find_all('tr'):

        try:
            row_name = each_row.th.text
            if row_name == 'Genes':
                gene_list = each_row.find_all('a')

                for each_gene in gene_list:

                    if '/dbget-bin/www_bget?' in each_gene.get('href'):

                        gene_name = each_gene.text
                        gene_link = 'https://www.genome.jp%s' % each_gene.get('href')
                        gene_soup = BeautifulSoup(get(gene_link).text, features="lxml")

                        for gene_soup_row in gene_soup.find_all('tr'):

                            try:
                                gene_soup_row_name = gene_soup_row.th.text

                                if gene_soup_row_name == 'AA seq':

                                    # remove button text
                                    for button in gene_soup_row.find_all('button'):
                                        button.clear()

                                    to_write_out = '>%s %s\n' % (gene_name, gene_soup_row.td.text)

                                    file_out_handle.write(to_write_out)

                            except:
                                pass
        except:
            pass

    file_out_handle.close()


def main_code(args):

    interested_ko_file  = args['ko']
    output_folder       = args['o']
    force_overwrite     = args['force']
    num_threads         = args['t']

    # create output folder
    if (os.path.isdir(output_folder) is True) and (force_overwrite is False):
        print('Output folder already exists, program exited!')
        exit()
    else:
        force_create_folder(output_folder)

    # store interested ko in list
    interested_ko_list = []
    for interested_ko in open(interested_ko_file):
        interested_ko_list.append(interested_ko.strip())

    list_for_multiple_arguments_download_ko_genes = []
    for interested_ko in interested_ko_list:
        interested_ko_fasta_output = '%s/%s.fasta' % (output_folder, interested_ko)
        list_for_multiple_arguments_download_ko_genes.append([interested_ko, interested_ko_fasta_output])

    pool = mp.Pool(processes=num_threads)
    pool.map(download_ko_genes, list_for_multiple_arguments_download_ko_genes)
    pool.close()
    pool.join()


if __name__ == '__main__':

    download_ko_genes_parser = argparse.ArgumentParser()

    # arguments for COG_parser
    download_ko_genes_parser.add_argument('-ko',              required=True,                              help='interested KOs at level D, one id per line')
    download_ko_genes_parser.add_argument('-o',               required=False,                             help='output folder')
    download_ko_genes_parser.add_argument('-force',           required=False, action='store_true',        help='force overwrite')
    download_ko_genes_parser.add_argument('-t',               required=False, type=int, default=1,        help='number of threads')

    args = vars(download_ko_genes_parser.parse_args())
    main_code(args)



import os
import glob
import shutil
import argparse
from Bio import SeqIO
from datetime import datetime
from Bio.SeqRecord import SeqRecord
import multiprocessing as mp
from BioSAK.BioSAK_config import config_dict


COG_parser_usage = '''
====================================== COG2003 example commands ======================================

# module needed
module load python/3.7.3
module load perl/5.20.1
module load blast+/2.6.0

# annotate protein sequences
BioSAK COG2003 -m P -t 6 -db_dir /srv/scratch/z5039045/DB/COG_DB -i recipient.faa
BioSAK COG2003 -m P -t 6 -db_dir /srv/scratch/z5039045/DB/COG_DB -i faa_files -x faa

# annotate DNA sequences
BioSAK COG2003 -m N -t 6 -db_dir /srv/scratch/z5039045/DB/COG_DB -i recipient.ffn
BioSAK COG2003 -m N -t 6 -db_dir /srv/scratch/z5039045/DB/COG_DB -i ffn_files -x ffn

# Prepare DB files:
cd $db_dir
wget ftp://ftp.ncbi.nih.gov/pub/mmdb/cdd/little_endian/Cog_LE.tar.gz
wget ftp://ftp.ncbi.nlm.nih.gov/pub/mmdb/cdd/cddid.tbl.gz
wget ftp://ftp.ncbi.nih.gov/pub/COG/COG/fun.txt
wget ftp://ftp.ncbi.nih.gov/pub/COG/COG/whog
gunzip Cog_LE.tar.gz
gunzip cddid.tbl.gz

# How it works:
https://github.com/aleimba/bac-genomics-scripts/tree/master/cdd2cog

======================================================================================================
'''


def force_create_folder(folder_to_create):
    if os.path.isdir(folder_to_create):
        shutil.rmtree(folder_to_create, ignore_errors=True)
        if os.path.isdir(folder_to_create):
            shutil.rmtree(folder_to_create, ignore_errors=True)
            if os.path.isdir(folder_to_create):
                shutil.rmtree(folder_to_create, ignore_errors=True)
                if os.path.isdir(folder_to_create):
                    shutil.rmtree(folder_to_create, ignore_errors=True)

    os.mkdir(folder_to_create)


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_ext = os.path.splitext(file_name)

    return file_path, file_basename, file_ext


def dna2aa(dna_file, aa_file):
    query_aa_handle = open(aa_file, 'w')
    for each in SeqIO.parse(dna_file, 'fasta'):
        each_aa = each.seq.translate()
        each_aa_record = SeqRecord(each_aa)
        each_aa_record.id = each.id
        each_aa_record.description = each.description
        SeqIO.write(each_aa_record, query_aa_handle, 'fasta')
    query_aa_handle.close()


def Annotation_COG_worker(argument_list):

    pwd_input_file = argument_list[0]
    pwd_db =         argument_list[1]
    pwd_cdd2cog =    argument_list[2]
    pwd_cddid =      argument_list[3]
    pwd_fun =        argument_list[4]
    pwd_whog =       argument_list[5]
    sequence_type =  argument_list[6]
    output_folder =  argument_list[7]

    input_seq_no_path, input_seq_no_ext, input_seq_ext = sep_path_basename_ext(pwd_input_file)
    current_output_folder = '%s/%s_COG_wd' % (output_folder, input_seq_no_ext)

    rpsblast_output = '%s_COG.tab' % (input_seq_no_ext)
    pwd_rpsblast_output = '%s/%s_COG.tab' % (current_output_folder, input_seq_no_ext)

    force_create_folder(current_output_folder)


    input_seq_aa = ''
    if (sequence_type == 'N') or (sequence_type == 'n'):
        input_seq_aa = '%s_aa.fasta' % input_seq_no_ext
        dna2aa(pwd_input_file, input_seq_aa)
    elif (sequence_type == 'P') or (sequence_type == 'p'):
        input_seq_aa = pwd_input_file
    else:
        print('Specified input sequence type unrecognizable, program exited!')
        exit()

    # run rpsblast
    os.system('rpsblast -query %s -db %s -out %s -evalue 1e-2 -outfmt 6 -num_threads 1' % (input_seq_aa, pwd_db, pwd_rpsblast_output))

    # run cdd2cog.perl
    current_wd = os.getcwd()
    os.chdir(current_output_folder)
    os.system('perl %s -r %s -c %s -f %s -w %s' % (pwd_cdd2cog, rpsblast_output, pwd_cddid, pwd_fun, pwd_whog))
    os.chdir(current_wd)

    # rename output files
    each_op_file_re = '%s/results/*.txt' % current_output_folder
    each_op_file_list = [os.path.basename(file_name) for file_name in glob.glob(each_op_file_re)]
    for each_op_file in each_op_file_list:

        pwd_each_op_file = '%s/results/%s' % (current_output_folder, each_op_file)
        each_op_file_renamed = '%s_%s' % (input_seq_no_ext, each_op_file)
        pwd_each_op_file_renamed = '%s/%s' % (current_output_folder, each_op_file_renamed)

        os.system('mv %s %s' % (pwd_each_op_file, pwd_each_op_file_renamed))

    os.system('rm -r %s/results' % current_output_folder)


def get_COG_annot_df(annotation_dir, stats_level, annotation_df_absolute_num, annotation_df_percentage):

    annotation_dir_re = '%s/*_COG_wd' % annotation_dir
    annotation_folder_list = [os.path.basename(file_name) for file_name in glob.glob(annotation_dir_re)]

    cog_num_dict = {}
    all_identified_cog = set()
    for annotation_folder in annotation_folder_list:

        annotation_folder_basename = annotation_folder.split('_COG_wd')[0]

        pwd_annotation_stats_file = ''
        if stats_level == 'cog_id':
            pwd_annotation_stats_file = '%s/%s/%s_cog_stats.txt' % (annotation_dir, annotation_folder, annotation_folder_basename)
        if stats_level == 'cog_cate':
            pwd_annotation_stats_file = '%s/%s/%s_func_stats.txt' % (annotation_dir, annotation_folder, annotation_folder_basename)

        current_cog_to_num_dict = {}
        for cog in open(pwd_annotation_stats_file):
            ko_split = cog.strip().split('\t')
            current_cog_to_num_dict[ko_split[0]] = int(ko_split[2])
            all_identified_cog.add(ko_split[0])

        cog_num_dict[annotation_folder_basename] = current_cog_to_num_dict

    all_identified_cog_list = sorted([i for i in all_identified_cog])

    annotation_df_absolute_num_handle = open(annotation_df_absolute_num, 'w')
    annotation_df_percentage_handle = open(annotation_df_percentage, 'w')
    annotation_df_absolute_num_handle.write('\t%s\n' % '\t'.join(all_identified_cog_list))
    annotation_df_percentage_handle.write('\t%s\n' % '\t'.join(all_identified_cog_list))
    for annotation_folder in sorted(annotation_folder_list):

        annotation_folder_basename = annotation_folder.split('_COG_wd')[0]
        current_cog_num_dict = cog_num_dict[annotation_folder_basename]

        current_cog_num_list = []
        for identified_cog in all_identified_cog_list:

            # get num list
            identified_cog_num = 0
            if identified_cog in current_cog_num_dict:
                identified_cog_num = current_cog_num_dict[identified_cog]
            current_cog_num_list.append(identified_cog_num)

        # get percentage
        current_cog_num_sum = sum(current_cog_num_list)
        current_cog_num_list_percentage = [float("{0:.2f}".format(i * 100 / current_cog_num_sum)) for i in current_cog_num_list]

        # write out
        annotation_df_absolute_num_handle.write('%s\t%s\n' % (annotation_folder_basename, '\t'.join([str(i) for i in current_cog_num_list])))
        annotation_df_percentage_handle.write('%s\t%s\n' % (annotation_folder_basename, '\t'.join([str(i) for i in current_cog_num_list_percentage])))

    annotation_df_absolute_num_handle.close()
    annotation_df_percentage_handle.close()


def COG2003(args, config_dict):

    file_in =           args['i']
    file_extension =    args['x']
    sequence_type =     args['m']
    DB_dir =            args['db_dir']
    num_threads =       args['t']

    pwd_cdd2cog = config_dict['cdd2cog_perl']

    pwd_whog =      '%s/whog'       % DB_dir
    pwd_cddid =     '%s/cddid.tbl'  % DB_dir
    pwd_db =        '%s/Cog'        % DB_dir
    pwd_fun =       '%s/fun.txt'    % DB_dir

    time_format = '[%Y-%m-%d %H:%M:%S] '


    ########################################### check input is file or folder ##########################################

    # if input is file
    if os.path.isfile(file_in) is True:

        file_in_path, file_in_basename, file_in_ext = sep_path_basename_ext(file_in)

        Annotation_COG_worker([file_in, pwd_db, pwd_cdd2cog, pwd_cddid, pwd_fun, pwd_whog, sequence_type, file_in_path])

    # if input is folder
    else:

        ############################################# check whether file exist #############################################

        # check whether db file exist
        unfound_inputs = []
        for each_input in [pwd_fun, pwd_cddid, pwd_whog]:
            if (not os.path.isfile(each_input)) and (not os.path.isdir(each_input)):
                unfound_inputs.append(each_input)
        if len(unfound_inputs) > 0:
            for each_unfound in unfound_inputs:
                print('%s not found' % each_unfound)
            exit()


        # check whether input folder exist
        if os.path.isdir(file_in) is False:
            print(datetime.now().strftime(time_format) + 'input folder not found, program exited')
            exit()
        else:
            # check whether input genome exist
            input_file_re = '%s/*.%s' % (file_in, file_extension)
            input_file_name_list = [os.path.basename(file_name) for file_name in glob.glob(input_file_re)]

            if len(input_file_name_list) == 0:
                print(datetime.now().strftime(time_format) + 'input file not found, program exited')
                exit()


            ################################################### define file name ###################################################

            if '/' in file_in:
                file_in_folder_name = file_in.split('/')[-1]
            else:
                file_in_folder_name = file_in

            output_folder = '%s_COG_wd' % file_in_folder_name

            # create output folder
            force_create_folder(output_folder)

            annotation_df_cog_id_num =          '%s/%s_cog_id_num.txt'           % (output_folder, file_in_folder_name)
            annotation_df_cog_id_percent =      '%s/%s_cog_id_percentage.txt'    % (output_folder, file_in_folder_name)
            annotation_df_cog_cate_num =        '%s/%s_cog_cate_num.txt'         % (output_folder, file_in_folder_name)
            annotation_df_cog_cate_percent =    '%s/%s_cog_cate_percentage.txt'  % (output_folder, file_in_folder_name)


            ######################################################### main #########################################################

            print(datetime.now().strftime(time_format) + 'Running COG annotation for %s input files with %s cores' % (len(input_file_name_list), num_threads))

            list_for_multiple_arguments_COG = []
            for input_file in input_file_name_list:

                pwd_input_file = '%s/%s' % (file_in, input_file)
                list_for_multiple_arguments_COG.append([pwd_input_file, pwd_db, pwd_cdd2cog, pwd_cddid, pwd_fun, pwd_whog, sequence_type, output_folder])

            # run COG annotaion files with multiprocessing
            pool = mp.Pool(processes=num_threads)
            pool.map(Annotation_COG_worker, list_for_multiple_arguments_COG)
            pool.close()
            pool.join()


            # get dataframe
            get_COG_annot_df(output_folder, 'cog_id', annotation_df_cog_id_num, annotation_df_cog_id_percent)
            get_COG_annot_df(output_folder, 'cog_cate', annotation_df_cog_cate_num, annotation_df_cog_cate_percent)


        ################################################## Final report ####################################################

    print(datetime.now().strftime(time_format) + 'Done!')


if __name__ == '__main__':

    COG_parser = argparse.ArgumentParser()

    # arguments for COG_parser
    COG_parser.add_argument('-i',               required=True,  help='path to input sequences (in multi-fasta format)')
    COG_parser.add_argument('-x',               required=False, help='file extension')
    COG_parser.add_argument('-m',               required=True,  help='The type of input sequences, "N/n" for "nucleotide", "P/p" for "protein"')
    COG_parser.add_argument('-db_dir',          required=True,  help='folder holds Cog, whog, fun.txt and cddid.tbl')
    COG_parser.add_argument('-t',               required=False, type=int, default=1, help='number of threads')

    args = vars(COG_parser.parse_args())

    COG2003(args, config_dict)




# gtdb_op_txt                 = '/Users/songweizhi/Desktop/retrieve_GTDB_genomes/Sponge_MAGs_1677.ar122.summary.tsv'
# gtdb_ref_gnm_folder         = '/srv/scratch/z5039045/DB/GTDB_r202/release202/fastani'

gtdb_gnm_metadata           = '/Users/songweizhi/Desktop/retrieve_GTDB_genomes/ar122_metadata_r202.tsv'
gtdb_ref_gnm_path_txt       = '/Users/songweizhi/Desktop/retrieve_GTDB_genomes/GTDB_r202_genome_paths.tsv'
taxons_to_retrieve_txt      = '/Users/songweizhi/Desktop/retrieve_GTDB_genomes/taxons_to_retrieve.txt'



#!/usr/bin/env python3

import os
import glob
import pandas
import argparse
import numpy as np

'''

cd /Users/songweizhi/Desktop
Python3 ~/PycharmProjects/BioSAK/BioSAK/boxplot_matrix_dbCAN.py -in MAGs_dRep99_dbCAN_stats_GeneNumber -in_percent -skip_1st_row
Rscript ~/PycharmProjects/BioSAK/BioSAK/COG_boxplot_last1row.R -i MAGs_dRep99_dbCAN_stats_GeneNumber_CAZy_cate.txt -o MAGs_dRep99_dbCAN_stats_GeneNumber_CAZy_cate.png

'''


def cazy_id_to_cate(cazy_id):

    cazy_cate = ''
    stop_now = False
    for each_element in cazy_id:
        if stop_now is False:
            if each_element.isalpha() is True:
                cazy_cate += each_element
            else:
                stop_now = True

    return cazy_cate


def turn_to_percentage(number_list):
    number_list_percent = []
    for each_element in number_list:
        each_element_percent = float("{0:.2f}".format(each_element / sum(number_list)))
        number_list_percent.append(each_element_percent)
    return number_list_percent


def dict_to_file(genome_name_list, annotation_dict_cazy_cate, column_list_cazy_cate, in_percent, output_csv):
    # get category_num_lol
    category_num_lol = []
    for genome in genome_name_list:

        current_genome_annotation_results_dict = annotation_dict_cazy_cate[genome]

        current_genome_annotation_results_list = []
        for cog_cate in column_list_cazy_cate:
            cog_cate_num = 0
            if cog_cate in current_genome_annotation_results_dict:
                cog_cate_num = current_genome_annotation_results_dict[cog_cate]
            current_genome_annotation_results_list.append(cog_cate_num)

        # turn absolute number to percentage if specified
        if in_percent is True:
            current_genome_annotation_results_list_in_percent = turn_to_percentage(
                current_genome_annotation_results_list)
            category_num_lol.append(current_genome_annotation_results_list_in_percent)
        else:
            category_num_lol.append(current_genome_annotation_results_list)

    # turn list into arrary
    category_num_arrary = np.array(category_num_lol)

    # add row and column name to dataframe
    category_num_df = pandas.DataFrame(category_num_arrary, index=genome_name_list, columns=column_list_cazy_cate)

    # write out
    category_num_df.to_csv(output_csv)


parser = argparse.ArgumentParser()
parser.add_argument('-in',            required=True,                          help='folder holds the func_stats.txt file for each genome')
parser.add_argument('-in_percent',    required=False, action="store_true",    help='in percent')
parser.add_argument('-skip_1st_row',  required=False, action="store_true",    help='skip 1st (header) row in input files')

args = vars(parser.parse_args())
input_folder =                  args['in']
in_percent =                    args['in_percent']
skip_1st_row =                  args['skip_1st_row']

if input_folder[-1] == '/':
    input_folder = input_folder[:-1]

output_csv_cazy_id   = '%s_CAZy_id.txt'   % input_folder
output_csv_cazy_cate = '%s_CAZy_cate.txt' % input_folder


input_files = '%s/*.txt' % input_folder
file_list = [os.path.basename(file_name) for file_name in glob.glob(input_files)]
file_list = sorted(file_list)


genome_name_list = []
detected_cazy_id_list = set()
detected_cazy_cate_list = set()
annotation_results_dict_cazy_id = {}
annotation_results_dict_cazy_cate = {}
cate_to_description_dict = {}
for each_file in file_list:
    genome_name = '_'.join(each_file.split('_')[:-2])
    genome_name_list.append(genome_name)
    current_annotation_results_dict = {}
    current_annotation_results_dict_cazy_cate = {}
    n = 0
    for each_category in open('%s/%s' % (input_folder, each_file)):
        each_category_split = each_category.strip().split('\t')
        cazy_id =           each_category_split[0]
        cazy_cate = cazy_id_to_cate(cazy_id)
        category_num_str =      each_category_split[1]
        category_description =  each_category_split[2]

        if skip_1st_row is True:
            if n > 0:
                current_annotation_results_dict[cazy_id] = int(category_num_str)
                detected_cazy_id_list.add(cazy_id)

                if cazy_cate not in current_annotation_results_dict_cazy_cate:
                    current_annotation_results_dict_cazy_cate[cazy_cate] = int(category_num_str)
                else:
                    current_annotation_results_dict_cazy_cate[cazy_cate] += int(category_num_str)
                detected_cazy_cate_list.add(cazy_cate)
        else:
            current_annotation_results_dict[cazy_id] = int(category_num_str)
            detected_cazy_id_list.add(cazy_id)

            if cazy_cate not in current_annotation_results_dict_cazy_cate:
                current_annotation_results_dict_cazy_cate[cazy_cate] = int(category_num_str)
            else:
                current_annotation_results_dict_cazy_cate[cazy_cate] += int(category_num_str)
            detected_cazy_cate_list.add(cazy_cate)

        if cazy_id not in cate_to_description_dict:
            cate_to_description_dict[cazy_id] = category_description
        n += 1

    annotation_results_dict_cazy_id[genome_name] = current_annotation_results_dict
    annotation_results_dict_cazy_cate[genome_name] = current_annotation_results_dict_cazy_cate

column_order_list_cazy_id = sorted([i for i in detected_cazy_id_list])
column_order_list_cazy_cate = sorted([i for i in detected_cazy_cate_list])

dict_to_file(genome_name_list, annotation_results_dict_cazy_id, column_order_list_cazy_id, in_percent, output_csv_cazy_id)
dict_to_file(genome_name_list, annotation_results_dict_cazy_cate, column_order_list_cazy_cate, in_percent, output_csv_cazy_cate)











import os
import glob
import argparse
from Bio import SeqIO


get_gnm_size_parser_usage = '''
============= get_gnm_size example commands ============= 

BioSAK get_gnm_size -i genome_1.fa
BioSAK get_gnm_size -i refined_MAGs -x fasta 
BioSAK get_gnm_size -i refined_MAGs -x fasta -total

=========================================================
'''


def get_gnm_size(argument_list):

    mag_folder =        argument_list['i']
    mag_extension =     argument_list['x']
    get_total_size =    argument_list['total']


    if os.path.isfile(mag_folder) is True:

        total_len = 0
        for each_seq in SeqIO.parse(mag_folder, 'fasta'):
            total_len += len(each_seq.seq)

        # in Kbp
        total_len_Kbp = total_len/1024
        total_len_Kbp = float("{0:.2f}".format(total_len_Kbp))

        # in Mbp
        total_len_Mbp = total_len/(1024*1024)
        total_len_Mbp = float("{0:.2f}".format(total_len_Mbp))

        print('Genome\tlength(bp)\tlength(Kbp)\tlength(Mbp)')
        print('%s\t%s\t%s\t%s' % (mag_folder, total_len, total_len_Kbp, total_len_Mbp))


    elif os.path.isdir(mag_folder) is True:

        mag_file_re = '%s/*.%s' % (mag_folder, mag_extension)
        mag_file_list = [os.path.basename(file_name) for file_name in glob.glob(mag_file_re)]

        if len(mag_file_list) == 0:
            print('No file detected, program exited')
            exit()

        print('Genome\tlength(bp)\tlength(Kbp)\tlength(Mbp)')
        all_mag_size = 0
        for each_mag in sorted(mag_file_list):
            pwd_mag = '%s/%s' % (mag_folder, each_mag)
            total_len = 0
            for each_seq in SeqIO.parse(pwd_mag, 'fasta'):
                total_len += len(each_seq.seq)
            all_mag_size += total_len

            # in Kbp
            total_len_Kbp = total_len/1024
            total_len_Kbp = float("{0:.2f}".format(total_len_Kbp))

            # in Mbp
            total_len_Mbp = total_len/(1024*1024)
            total_len_Mbp = float("{0:.2f}".format(total_len_Mbp))

            print('%s\t%s\t%s\t%s' % (each_mag, total_len, total_len_Kbp, total_len_Mbp))

        if get_total_size is True:
            get_total_size_Kbp = all_mag_size/1024
            get_total_size_Mbp = all_mag_size/(1024*1024)
            get_total_size_Kbp = float("{0:.2f}".format(get_total_size_Kbp))
            get_total_size_Mbp = float("{0:.2f}".format(get_total_size_Mbp))
            print('Number of genome: %s, total size: %s bp, %s Kbp, %s Mbp' % (len(mag_file_list), all_mag_size, get_total_size_Kbp, get_total_size_Mbp))


if __name__ == '__main__':

    get_gnm_size_parser = argparse.ArgumentParser()
    get_gnm_size_parser.add_argument('-i',      required=True,                          help='MAG file/folder')
    get_gnm_size_parser.add_argument('-x',      required=False,                         help='file extension')
    get_gnm_size_parser.add_argument('-total',  required=False,  action='store_true',   help='get total size')
    args = vars(get_gnm_size_parser.parse_args())
    get_gnm_size(args)



import os
import argparse


js_cmds_usage = '''
========================= js_cmds example commands =========================

BioSAK js_cmds -p Test -cmd cmds.txt -header js_header.txt -n 3 -force

============================================================================
'''


def js_cmds(args):

    js_prefix       = args['p']
    cmd_file        = args['cmds']
    auto_submit     = args['auto']
    js_header_file  = args['header']
    cmd_num_per_js  = args['n']
    js_folder_hpc   = args['js_hpc']
    force_overwrite = args['force']

    pwd_js_folder       = '%s/%s_js' % (os.getcwd(), js_prefix)

    if js_folder_hpc is None:
        js_folder_hpc = pwd_js_folder


    if (os.path.isdir(pwd_js_folder) is True) and (force_overwrite is False):
        print('Output folder detected, program exited: %s' % pwd_js_folder)
        exit()
    else:
        if os.path.isdir(pwd_js_folder) is True:
            os.system('rm -r %s' % pwd_js_folder)
            os.mkdir(pwd_js_folder)
        else:
            os.mkdir(pwd_js_folder)

    # read in js header
    js_header_str = ''
    for each_line in open(js_header_file):
        js_header_str += each_line

    js_index = 1
    n = 1
    for each_cmd in open(cmd_file):

        pwd_js_previous = '%s/%s_%s.sh' % (pwd_js_folder, js_prefix, (js_index - 1))
        pwd_js          = '%s/%s_%s.sh' % (pwd_js_folder, js_prefix, js_index)
        pwd_js_hpc      = '%s/%s_%s.sh' % (js_folder_hpc, js_prefix, js_index)

        if os.path.isfile(pwd_js) is False:

            if js_index > 1:

                if auto_submit is True:
                    pwd_js_previous_handle = open(pwd_js_previous, 'a')
                    pwd_js_previous_handle.write('qsub %s\n' % pwd_js_hpc)
                    pwd_js_previous_handle.close()

            pwd_js_handle = open(pwd_js, 'w')
            pwd_js_handle.write(js_header_str)
            pwd_js_handle.write('\n')
            pwd_js_handle.write(each_cmd)
            pwd_js_handle.close()
        else:
            pwd_js_handle = open(pwd_js, 'a')
            pwd_js_handle.write(each_cmd)
            pwd_js_handle.close()

        if n % cmd_num_per_js == 0:
            js_index += 1
        n += 1


if __name__ == '__main__':

    parser = argparse.ArgumentParser()

    parser.add_argument('-p',      required=True,                       help='js prefix')
    parser.add_argument('-cmds',   required=True,                       help='cmds file')
    parser.add_argument('-auto',   required=False, action="store_true", help='automatically submit next js')
    parser.add_argument('-header', required=True,                       help='js header')
    parser.add_argument('-n',      required=True, type=int,             help='number of cmds per js')
    parser.add_argument('-js_hpc', required=False, default=None,        help='Full path to js folder on HPC')
    parser.add_argument('-force',  required=False, action="store_true", help='force overwrite existing results')

    args = vars(parser.parse_args())
    js_cmds(args)



import os
import glob
import math
import random
import argparse
import numpy as np
import seaborn as sns
from Bio import SeqIO
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
from matplotlib.patches import Patch


Plot_MAG_parser_usage = '''
====================================== Plot_MAG example commands ======================================

# annotate protein sequences
BioSAK Plot_MAG -i refined_MAGs -x fasta -d contig_depth.txt -o refined_MAGs.pdf

# depth file: tab separated, no header

======================================================================================================
'''


def get_color_list(color_num):

    if color_num <= 8:
        color_list_combined = ['#3787c0', '#39399f', '#ffb939', '#399f39', '#9f399f', '#fb694a', '#9f9f39', '#959595']

    elif 8 < color_num <= 16:
        color_list_combined = ['#2b7bba', '#89bedc', '#2e2e99', '#8a8acc', '#ffa500', '#ffc55c', '#2e992e', '#8acc8a', '#992e99', '#cc8acc', '#d52221', '#fc8161', '#99992e', '#cccc8a', '#5c5c5c', '#adadad']

    else:
        color_num_each = math.ceil(color_num/8) + 2

        color_list_1 = sns.color_palette('Blues',  n_colors=color_num_each).as_hex()
        color_list_2 = sns.light_palette('navy',   n_colors=color_num_each).as_hex()
        color_list_3 = sns.light_palette('orange', n_colors=color_num_each).as_hex()
        color_list_4 = sns.light_palette('green',  n_colors=color_num_each).as_hex()
        color_list_5 = sns.light_palette('purple', n_colors=color_num_each).as_hex()
        color_list_6 = sns.color_palette('Reds',   n_colors=color_num_each).as_hex()
        color_list_7 = sns.light_palette('olive',  n_colors=color_num_each).as_hex()
        color_list_8 = sns.color_palette('Greys', n_colors=color_num_each).as_hex()

        color_list_combined = []
        for color_list in [color_list_1, color_list_2, color_list_3, color_list_4, color_list_5, color_list_6, color_list_7, color_list_8]:
            for color in color_list[2:][::-1]:
                color_list_combined.append(color)

    color_list_to_return = random.sample(color_list_combined, color_num)

    color_list_to_return_sorted = []
    for color_to_return in color_list_combined:
        if color_to_return in color_list_to_return:
            color_list_to_return_sorted.append(color_to_return)

    return color_list_to_return_sorted


def Plot_MAG(argument_list):

    bin_folder =  argument_list['i']
    bin_ext =     argument_list['x']
    depth_file =  argument_list['d']
    output_plot = argument_list['o']

    # read in depth info
    ctg_depth_dict = dict()
    for each_ctg in open(depth_file):
        each_ctg_split = each_ctg.strip().split('\t')
        if not each_ctg.startswith('contigName	contigLen	totalAvgDepth	sample1	sample1-var'):
            ctg_depth_dict[each_ctg_split[0]] = float(each_ctg_split[1])

    # get mag file list
    bin_file_re = '%s/*.%s' % (bin_folder, bin_ext)
    bin_file_list = [os.path.basename(file_name) for file_name in glob.glob(bin_file_re)]

    if len(bin_file_list) == 0:
        print('No MAG detected, program exited')
        exit()

    color_list = get_color_list(len(bin_file_list))

    legend_elements = []
    num_list_depth = []
    num_list_gc = []
    ctg_color_list = []
    ctg_len_list = []
    index = 0
    for each_mag in bin_file_list:
        pwd_each_mag = '%s/%s' % (bin_folder, each_mag)
        for each_ctg in SeqIO.parse(pwd_each_mag, 'fasta'):
            ctg_depth = ctg_depth_dict[each_ctg.id]
            ctg_seq = str(each_ctg.seq).upper()
            ctg_gc = ((ctg_seq.count('G')) + (ctg_seq.count('C')))*100/len(ctg_seq)
            num_list_depth.append(ctg_depth)
            num_list_gc.append(ctg_gc)
            ctg_color_list.append(color_list[index])
            ctg_len_list.append(len(ctg_seq)/2000)
        legend_elements.append(Patch(facecolor=color_list[index], edgecolor=color_list[index], label=each_mag))
        index += 1

    num_array_depth = np.array(num_list_depth)
    num_array_gc = np.array(num_list_gc)
    num_array_len = np.array(ctg_len_list)

    plt.scatter(num_array_depth, num_array_gc, c=ctg_color_list, s=num_array_len, alpha=0.3, linewidths=0)
    #plt.scatter(num_array_depth, num_array_gc, c=ctg_color_list, s=5, alpha=0.6, linewidths=0)
    plt.title('Number of genome: %s' % len(bin_file_list), fontsize=9)
    plt.xlabel("Depth (X)", size=9)
    plt.ylabel("GC (%)", size=9)
    plt.legend(handles=legend_elements, loc='center left', prop={'size': 6}, bbox_to_anchor=(1, 0.5))
    plt.tight_layout()
    plt.savefig(output_plot)
    plt.close()


if __name__ == '__main__':

    Plot_MAG_parser = argparse.ArgumentParser()
    Plot_MAG_parser.add_argument('-i',   required=True,  help='MAG folder')
    Plot_MAG_parser.add_argument('-x',   required=True,  help='file extension')
    Plot_MAG_parser.add_argument('-d',   required=True,  help='contig depth')
    Plot_MAG_parser.add_argument('-o',   required=True,  help='output plot')
    args = vars(Plot_MAG_parser.parse_args())
    Plot_MAG(args)
